import { __assign, __extends, __makeTemplateObject, __rest } from "tslib";
import * as React from 'react';
import styled from 'styled-components';
import { darken } from '../../utils';
import { Box } from '../Box';
import { Flex } from '../Flex';
var ANGLERANGE = 135;
var getAngleBetweenPoints = function (p1, p2) {
    return (Math.atan2(p2.y - p1.y, p2.x - p1.x) * 180) / Math.PI;
};
var constrainNumber = function (value, lower, upper) {
    if (lower === void 0) { lower = 0; }
    if (upper === void 0) { upper = 100; }
    return Math.min(Math.max(value, lower), upper);
};
var StyledSVG = styled.svg(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n\tdisplay: block;\n\twidth: 100%;\n\theight: 100%;\n"], ["\n\tdisplay: block;\n\twidth: 100%;\n\theight: 100%;\n"])));
var ArcSliderContainer = styled(Box)(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n\tposition: relative;\n"], ["\n\tposition: relative;\n"])));
var LabelContainer = styled(Flex).attrs({
    justifyContent: 'center',
    flexDirection: 'column',
})(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n\tposition: absolute;\n\ttop: 0;\n\tright: 0;\n\tbottom: 0;\n\tleft: 0;\n\ttext-align: center;\n\tpointer-events: none;\n"], ["\n\tposition: absolute;\n\ttop: 0;\n\tright: 0;\n\tbottom: 0;\n\tleft: 0;\n\ttext-align: center;\n\tpointer-events: none;\n"])));
var polarToCartesian = function (centerX, centerY, radius, angleInDegrees) {
    var angleInRadians = ((angleInDegrees - 90) * Math.PI) / 180.0;
    return {
        x: centerX + radius * Math.cos(angleInRadians),
        y: centerY + radius * Math.sin(angleInRadians),
    };
};
var describeArc = function (x, y, radius, startAngle, endAngle) {
    var start = polarToCartesian(x, y, radius, endAngle);
    var end = polarToCartesian(x, y, radius, startAngle);
    var largeArcFlag = endAngle - startAngle <= 180 ? '0' : '1';
    var d = "M " + start.x + " " + start.y + " A " + radius + " " + radius + " 0 " + largeArcFlag + " 0 " + end.x + " " + end.y;
    return d;
};
var ArcSlider = (function (_super) {
    __extends(ArcSlider, _super);
    function ArcSlider(props) {
        var _this = _super.call(this, props) || this;
        _this.$slider = null;
        _this.handleMouseUp = function () {
            _this.setState({ isDragging: false });
        };
        _this.handleMouseDown = function (e) {
            e.preventDefault();
            _this.setState({ isDragging: true });
        };
        _this.handleMouseMove = function (e) {
            if (_this.state.isDragging && _this.$slider) {
                var _a = _this.$slider.getBoundingClientRect(), left = _a.left, top_1 = _a.top, width = _a.width, height = _a.height;
                var sliderCenter = {
                    x: left + width / 2,
                    y: top_1 + height / 2,
                };
                var mouseLocation = {
                    x: e.pageX,
                    y: e.pageY,
                };
                if (!!e.touches) {
                    mouseLocation = {
                        x: e.touches[0].pageX,
                        y: e.touches[0].pageY,
                    };
                }
                var angleDegrees = getAngleBetweenPoints(sliderCenter, mouseLocation) + 90;
                if (angleDegrees > 180) {
                    angleDegrees = Math.max(angleDegrees - 360, -ANGLERANGE);
                }
                else if (angleDegrees < -ANGLERANGE) {
                    angleDegrees = -ANGLERANGE;
                }
                else if (angleDegrees > ANGLERANGE) {
                    angleDegrees = ANGLERANGE;
                }
                var value = (angleDegrees + ANGLERANGE) / (ANGLERANGE * 2);
                _this.setState({
                    innerValue: value,
                });
                if (_this.props.onValueChange) {
                    _this.props.onValueChange(value);
                }
            }
        };
        var value = props.value;
        _this.state = {
            isDragging: false,
            innerValue: value == null ? 0 : constrainNumber(value, 0, 1),
        };
        return _this;
    }
    ArcSlider.prototype.componentDidMount = function () {
        document.addEventListener('mousemove', this.handleMouseMove);
        document.addEventListener('touchmove', this.handleMouseMove);
        document.addEventListener('mouseup', this.handleMouseUp);
        document.addEventListener('touchend', this.handleMouseUp);
        document.addEventListener('touchcancel', this.handleMouseUp);
    };
    ArcSlider.prototype.componentWillUnmount = function () {
        document.removeEventListener('mousemove', this.handleMouseMove);
        document.removeEventListener('touchmove', this.handleMouseMove);
        document.removeEventListener('mouseup', this.handleMouseUp);
        document.removeEventListener('touchend', this.handleMouseUp);
        document.removeEventListener('touchcancel', this.handleMouseUp);
    };
    ArcSlider.prototype.render = function () {
        var _this = this;
        var innerValue = this.state.innerValue;
        var value = this.props.value == null
            ? innerValue
            : constrainNumber(this.props.value, 0, 1);
        var _a = this.props, onValueChange = _a.onValueChange, children = _a.children, props = __rest(_a, ["onValueChange", "children"]);
        var color = this.props.fillColor || '#FEC400';
        var background = this.props.background || '#707274';
        var valueAngle = ANGLERANGE * 2 * value - ANGLERANGE;
        return (React.createElement(ArcSliderContainer, __assign({}, props),
            React.createElement(StyledSVG, { style: {
                    cursor: this.state.isDragging ? 'grabbing' : 'grab',
                }, ref: function (elem) { return (_this.$slider = elem); }, viewBox: "0 0 600 600", onMouseDown: this.handleMouseDown, onTouchStart: this.handleMouseDown },
                React.createElement("defs", null,
                    React.createElement("linearGradient", { id: "rendition-arc-slider-gradient", x1: "0%", y1: "0%", x2: "100%", y2: "0%" },
                        React.createElement("stop", { offset: "0%", stopColor: darken(color) }),
                        React.createElement("stop", { offset: "100%", stopColor: color })),
                    React.createElement("filter", { xmlns: "http://www.w3.org/2000/svg", id: "rendition-arc-slider-dropshadow", height: "130%" },
                        React.createElement("feGaussianBlur", { in: "SourceAlpha", stdDeviation: "1.25" }),
                        React.createElement("feOffset", { dx: "0", dy: "0", result: "offsetblur" }),
                        React.createElement("feMerge", null,
                            React.createElement("feMergeNode", null),
                            React.createElement("feMergeNode", { in: "SourceGraphic" })))),
                React.createElement("path", { fill: "none", stroke: background, strokeWidth: "20.5", strokeLinecap: "round", d: describeArc(300, 300, 200, -ANGLERANGE, ANGLERANGE) }),
                React.createElement("path", { fill: "none", stroke: "url(#rendition-arc-slider-gradient)", strokeWidth: "22", strokeLinecap: "round", d: describeArc(300, 300, 200, -ANGLERANGE, valueAngle) }),
                React.createElement("circle", { cx: "300", cy: "100", r: "20", fill: "white", filter: "url(#rendition-arc-slider-dropshadow)", transform: "rotate(" + valueAngle + ", 300, 300)" }),
                React.createElement("circle", { cx: "300", cy: "100", r: "14", fill: color, transform: "rotate(" + valueAngle + ", 300, 300)" })),
            React.createElement(LabelContainer, null, children)));
    };
    return ArcSlider;
}(React.Component));
export default ArcSlider;
var templateObject_1, templateObject_2, templateObject_3;
